"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compile = void 0;
const generic_tokenizer_1 = __importDefault(require("../generic-tokenizer"));
const isDeclarationBlock = (blockCount, blockContent) => {
    if (blockCount < 2 && blockContent.includes('import')) {
        return true;
    }
    return false;
};
function compile(abellTemplate, filename = '<anonymous>.abell') {
    const tokenSchema = {
        COMMENTED_OUT_BLOCK_START: /\\{{/,
        BLOCK_START: /{{/,
        BLOCK_END: /}}/
    };
    const tokens = generic_tokenizer_1.default(abellTemplate, tokenSchema, 'default');
    let isInsideAbellBlock = false;
    let htmlCode = '';
    let blockCode = '';
    let declarations = '';
    let blockCount = 0;
    for (const token of tokens) {
        if (token.type === 'BLOCK_START') {
            isInsideAbellBlock = true;
            blockCount++;
        }
        else if (token.type === 'BLOCK_END') {
            isInsideAbellBlock = false;
            if (isDeclarationBlock(blockCount, blockCode)) {
                declarations = blockCode;
            }
            else {
                // JS Code
                htmlCode += `\${e(${blockCode})}`;
            }
            blockCode = '';
        }
        else {
            // normal string;
            if (isInsideAbellBlock) {
                blockCode += token.text;
            }
            else {
                htmlCode += token.text;
            }
        }
    }
    // console.log({ declarations });
    // console.log({ htmlString: htmlCode });
    const jsOut = `
  ${declarations}
  const e = (val) => {
    if (typeof val === 'function') {
      return val();
    }

    if (!val) {
      return '';
    }

    return val;
  };
  export const html = (props = {}) => {
    const Abell = { 
      props, 
      console: {
        log: (...args) => {
          console.log('\\u001b[90m[${filename}]:\\u001b[39m', ...args);
          return '';
        }
      }
    };
    return \`${htmlCode.trim()}\`
  };
  export default html;
  `;
    return jsOut;
}
exports.compile = compile;
exports.default = compile;
